/*M.A. Alberti, S. Baldan, P. Pasteris
  Progetto multimediale, Comunicazione digitale, 
  Università degli Studi di Milano 14/15
  Composizione di un'img layer con chiave cromatica e una di sfondo 
  Chroma Key usa lo spazio colore HSB
*/
import java.awt.Color;


PImage layer,backgr,img_B_W,img_alfa;

float targetHue;
float Htollerance,
      Stollerance,
      Vtollerance;

void setup() {
  // Processing supporta immagini formato: gif, jpg e png
  // si usa loadImage(id) per caricare una immagine esterna e 
  // createImage (w, h, format) per generarne una 
  backgr = loadImage("laboratorio_analisi.jpg");
  layer = loadImage("scienziata.jpg");
  img_B_W=createImage(layer.width, layer.height, RGB);
  img_alfa=createImage(layer.width, layer.height, ARGB);
  //valori presi con gimp e normalizzati tra 0 e 1
  targetHue=134f/360f; // cerchiamo la chiave cromatica verde
  //targetHue=45f/360f;
  Htollerance=0.15f;
  Stollerance=0.3f;
  Vtollerance=0.3f;
  
  // la mascherina per la visualizzazione delle 3 img computate
  // larga come 3 immagini e alta come img considerata
  size(backgr.width*3, backgr.height);
  noLoop();
  background(255); // background di colore bianco
}

void draw() {
  //elaborazione dell'img layer e composizione con l'img backgr di sfondo 
  chromaKey(layer, backgr);
  //I pos della mascherina: immagine composta 
  image(backgr, 0, 0);
  //II pos della mascherina: immagine B/N (bianco il layer e nero lo sfondo)
  image(img_B_W, img_B_W.width, 0);
  //III pos della mascherina: immagine layer con sfondo trasparente
  image(img_alfa, img_alfa.width*2, 0);
  //e salva immagine composta in formato PNG su file esterno
  savePNG(backgr);
}

void chromaKey(PImage layer, PImage backgr) {
  int dimension = layer.width*layer.height;
  int r,g,b;
  float[] hsb = new float[3];
  float deltaHue, alpha_channel;
    
  backgr.loadPixels();
  layer.loadPixels();
  img_B_W.loadPixels();
  img_alfa.loadPixels();
  
  for(int i=0;i<dimension;i++){
    //per ogni pixel dell'immagine layer converte dallo spazio colore RGB a HSB
    Color.RGBtoHSB((int)red(layer.pixels[i]),(int)green(layer.pixels[i]),(int)blue(layer.pixels[i]), hsb);
    //differenza di tonalità tra la tonalità cercata e quella corrente
    deltaHue = Math.abs(hsb[0]-targetHue);
    //decisione se il pixel è dello sfondo o dell'img
    if(deltaHue < Htollerance && hsb[1]> Stollerance && hsb[2]> Vtollerance){
      alpha_channel=0;   //è un pixel dell'img sfondo nella chiave cromatica   
    }else{
      alpha_channel=1;   //è un pixel dell'img layer da tenere
    }
    //costruisco il valore rgb del pixel del layer o dello sfondo
    r=(int)(alpha_channel*red(layer.pixels[i])+(1-alpha_channel)*red(backgr.pixels[i]));
    g=(int)(alpha_channel*green(layer.pixels[i])+(1-alpha_channel)*green(backgr.pixels[i]));
    b=(int)(alpha_channel*blue(layer.pixels[i])+(1-alpha_channel)*blue(backgr.pixels[i]));
    
    //I immagine
    //sovrappongo l'immagine layer allo sfondo; ottengo l'immagine obiettivo
    backgr.pixels[i]=color(r,g,b);
    
    //II immagine
    //immagine bianco/nero, nero lo sfondo, bianco il layer
    img_B_W.pixels[i] = color(alpha_channel*255, alpha_channel*255, alpha_channel*255);
    
    //III immagine
    //immagine ricostruita con l'aggiunta del canale alfa
    img_alfa.pixels[i]=color(r,g,b,alpha_channel*255);
  }
  backgr.updatePixels();
  img_B_W.updatePixels();
  img_alfa.updatePixels();
}
void savePNG(PImage img){
  img.save("img_composta.png");
}
